@@ -220,16 +220,18 @@ compare_uri_cb (gconstpointer element_data, gconstpointer data)
 {
 	EogImage *image;
 	GnomeVFSURI *uri;
-
+	GnomeVFSURI *image_uri;
+	gboolean equal;
+	
 	image = EOG_IMAGE (element_data);
 	uri = (GnomeVFSURI*) data;
+	image_uri = eog_image_get_uri (image);
 
-	if (gnome_vfs_uri_equal (eog_image_get_uri (image), uri)) {
-		return 0;
-	} 
-	else {
-		return 1;
-	}
+	equal = gnome_vfs_uri_equal (image_uri, uri);
+
+	gnome_vfs_uri_unref (image_uri);
+
+	return !equal;
 }
 
 static int
@@ -401,41 +403,6 @@ is_supported_mime_type (const char *mime_type)
 
 /* ================== Directory Loading stuff ===================*/
 
-
-static void 
-cleanup_dead_files (EogImageList *list)
-{
-	gint position = 0;
-	GList *node = NULL, *remove = NULL;
-	gboolean found = FALSE;
-	
-	if (list && list->priv)
-		node = list->priv->store;
-
-	while (node != NULL && !found) {
-		GnomeVFSURI *uri = eog_image_get_uri(node->data);
-		if (!gnome_vfs_uri_exists (uri)) {
-			remove = g_list_append (remove, GINT_TO_POINTER (position));
-		}	
-		node = node->next;		
-		position++;
-	}
-
-	while (remove) {
-		gint pos = GPOINTER_TO_INT (remove->data);
-		EogIter *iter = eog_image_list_get_iter_by_pos (list, pos);
-
-		if (iter) {
-			remove_image_private (list, iter);
-			g_signal_emit (list, eog_image_list_signals[IMAGE_REMOVED], 0, pos);
-		}
-		
-		remove = g_list_next (remove);
-	}
-	
-	g_list_free (remove);
-}
-
 static gboolean
 get_uri_info (GnomeVFSURI *uri, GnomeVFSFileInfo *info)
 {
@@ -453,6 +420,38 @@ get_uri_info (GnomeVFSURI *uri, GnomeVFSFileInfo *info)
 	return (result == GNOME_VFS_OK);
 }
 
+/* Returns whether the given file is in the list. If found and image is not
+   NULL, sets it to the EogImage found. */
+static gboolean
+eog_image_list_has_file (EogImageList *list,
+			 const gchar *info_uri,
+			 EogImage **image)
+{
+	GList *node = NULL;
+	GnomeVFSURI *uri = NULL;
+	gboolean found = FALSE;
+	
+	if (list && list->priv) {
+		node = list->priv->store;
+		
+		while (node != NULL && !found) {
+			
+			uri = eog_image_get_uri(node->data);
+			gchar *str = gnome_vfs_uri_to_string 
+				(uri, GNOME_VFS_URI_HIDE_NONE);
+			found = strcmp (str, info_uri) == 0;
+			if (found && image) {
+				*image = EOG_IMAGE (g_object_ref (node->data));
+			}
+			g_free (str);
+			node = node->next;
+			gnome_vfs_uri_unref (uri);
+		}
+	}
+
+	return found;
+}
+
 static void
 vfs_monitor_dir_cb (GnomeVFSMonitorHandle *handle,
           const gchar *monitor_uri,
@@ -464,34 +463,26 @@ vfs_monitor_dir_cb (GnomeVFSMonitorHandle *handle,
 	GnomeVFSURI *uri = NULL;
 	GnomeVFSFileInfo *info = NULL;
 	EogImage *image = NULL;
-	GList *node=NULL;
 	gboolean found = FALSE;
-
-	cleanup_dead_files (list);
 	
 	switch (event_type) {
 		case GNOME_VFS_MONITOR_EVENT_CHANGED:
 		case GNOME_VFS_MONITOR_EVENT_METADATA_CHANGED:
-		case GNOME_VFS_MONITOR_EVENT_DELETED:
 		case GNOME_VFS_MONITOR_EVENT_STARTEXECUTING:
 		case GNOME_VFS_MONITOR_EVENT_STOPEXECUTING:
 			break;
 			
-		case GNOME_VFS_MONITOR_EVENT_CREATED:
-		
-			if (list && list->priv) {
-				node = list->priv->store;
-				
-				while (node != NULL && !found) {
-			
-					uri = eog_image_get_uri(node->data);
-					gchar *str = gnome_vfs_uri_to_string 
-						(uri, GNOME_VFS_URI_HIDE_NONE);
-					found = (strcmp (str, info_uri)==0)?TRUE:FALSE;
-					g_free (str);
-					node = node->next;
-				}
+		case GNOME_VFS_MONITOR_EVENT_DELETED:
+			found = eog_image_list_has_file (list, info_uri, &image);
+			if (found) {
+				eog_image_list_remove_image (list, image);
+				g_object_unref (image);
 			}
+			break;
+
+		case GNOME_VFS_MONITOR_EVENT_CREATED:
+
+			found = eog_image_list_has_file (list, info_uri, NULL);
 			
 			if (!found) {
 				info = gnome_vfs_file_info_new ();
@@ -631,30 +622,30 @@ eog_image_list_add_uris (EogImageList *list, GList *uri_list)
 		else if (info->type == GNOME_VFS_FILE_TYPE_REGULAR && 
 			 g_list_length (uri_list) == 1) {
 
-			if (GNOME_VFS_FILE_INFO_LOCAL(info)) {	
-				/* Store the URI for initial image assignment */
-				initial_uri = gnome_vfs_uri_dup (uri); 
-				
-				uri = gnome_vfs_uri_get_parent (uri);
-				
-				if (!get_uri_info (uri, info))
-					continue;
+			/* Store the URI for initial image assignment */
+			initial_uri = gnome_vfs_uri_dup (uri); 
+			
+			uri = gnome_vfs_uri_get_parent (uri);
+			
+			if (!get_uri_info (uri, info))
+				continue;
 		
+			if (info->type == GNOME_VFS_FILE_TYPE_DIRECTORY) {
 				add_directory (list, uri, info);
 
-				/* If the file we explicitly requested wasn't 'found' when
-				 * scanning the directory then explicitly add it.
-				 */
-				if (NULL == g_list_find_custom (priv->store, initial_uri,
-								compare_uri_cb))
-				{
-					if (!get_uri_info (initial_uri, info))
-						continue;
+				gnome_vfs_uri_unref (uri);
+			}
 
-					add_regular (list, initial_uri, info);
-				}
-			} else {
-				add_regular (list, uri, info);
+			/* If the file we explicitly requested wasn't 'found' when
+			 * scanning the directory then explicitly add it.
+			 */
+			if (NULL == g_list_find_custom (priv->store, initial_uri,
+							compare_uri_cb))
+			{
+				if (!get_uri_info (initial_uri, info))
+					continue;
+
+				add_regular (list, initial_uri, info);
 			}
 		}	
 		else if (info->type == GNOME_VFS_FILE_TYPE_REGULAR && 
